@@ -65,7 +65,8 @@ class Hub:
         self._reverse_sub = None
         self._iter_ind = 0
         # Set up the conf OPT structure so it is always available
-        self.OPT = {}
+        self.OPT = self.pop.data.null_map()
+        self._imports = {}
 
     @property
     def _iter_subs(self):
@@ -76,6 +77,7 @@ class Hub:
             reverse_sub=self._reverse_sub,
             subs=self._subs,
             OPT=copy.copy(self.OPT),
+            imports={subname: mod.__name__ for subname, mod in self._imports.items()},
         )
 
     def __setstate__(self, state: Dict):
@@ -84,6 +86,10 @@ class Hub:
         opt = state["OPT"]
         reverse_sub = state["reverse_sub"]
 
+        self._imports = {
+            subname: importlib.import_module(modname)
+            for subname, modname in state["imports"].items()
+        }
         self.OPT = self.pop.data.imap(opt)
         self._restore_subs(subs, self)
         if reverse_sub:
@@ -177,10 +183,12 @@ class Hub:
             resolved = self._sub_alias[item]
             if resolved in self._subs:
                 return self._subs[resolved]
+        elif item in self._imports:
+            return self._imports[item]
         return self.__getattribute__(item)
 
     def __getitem__(self, item: str):
-        return getattr(self, item)
+        return getattr(self, str(item))
 
 
 class Sub:
@@ -270,6 +278,7 @@ class Sub:
         self._stop_on_failures = stop_on_failures
         self._is_contract = is_contract
         self._process_init = init
+        self._imports = {}
         self._prepare()
 
     def _prepare(self):
@@ -366,6 +375,7 @@ class Sub:
             default_recursive_contracts=self._default_recursive_contracts,
             _subs=self._subs,
             _reverse_sub=self._reverse_sub,
+            imports={subname: mod.__name__ for subname, mod in self._imports.items()},
         )
 
     def __setstate__(self, state: Dict):
@@ -374,6 +384,11 @@ class Sub:
         """
         self._subs = state.pop("_subs")
         self._reverse_sub = state.pop("_reverse_sub")
+        imports = state.pop("imports", {})
+        self._imports = {
+            subname: importlib.import_module(modname)
+            for subname, modname in imports.items()
+        }
         # This will have all the arguments needed to recreate the Sub with pop.sub.add
         self._restoreable_state = state
 
@@ -402,6 +417,8 @@ class Sub:
             resolved = self._sub_alias[item]
             if resolved in self._subs:
                 return self._subs[resolved]
+        elif item in self._imports:
+            return self._imports[item]
 
         mod = self._find_mod(item)
         if mod is None:
@@ -412,7 +429,7 @@ class Sub:
         return mod
 
     def __getitem__(self, item: str):
-        return getattr(self, item)
+        return getattr(self, str(item))
 
     def __contains__(self, item: str):
         try:
@@ -558,16 +575,14 @@ class Sub:
         contracts = pop.contract.load_contract(
             self._contracts, self._default_contracts, mod, vret["name"]
         )
-        recursive_contracts = set(
-            pop.contract.load_contract(
-                self._recursive_contracts,
-                self._default_recursive_contracts,
-                mod,
-                vret["name"],
-            )
+        tmp_recursive_contracts = pop.contract.load_contract(
+            self._recursive_contracts,
+            self._default_recursive_contracts,
+            mod,
+            vret["name"],
         )
         if getattr(self._root, "_recursive_contracts", None):
-            recursive_contracts.update(
+            tmp_recursive_contracts.extend(
                 pop.contract.load_contract(
                     self._root._recursive_contracts,
                     self._root._default_recursive_contracts,
@@ -575,7 +590,10 @@ class Sub:
                     vret["name"],
                 )
             )
-        recursive_contracts = list(recursive_contracts)
+        recursive_contracts = []
+        for contract in tmp_recursive_contracts:
+            if contract not in recursive_contracts:
+                recursive_contracts.append(contract)
         name = vret["name"]
         if name.endswith(EXT_SUFFIXES):
             for ext in EXT_SUFFIXES:
